/*
    DFRegTool.h
    Copyright 1997-2000, Deep Fried Software. All rights reserved
*/

#if !defined(AFX_REGTOOL_H__EBCBF331_598F_11D2_A553_000502DD3AA8__INCLUDED_)
#define AFX_REGTOOL_H__EBCBF331_598F_11D2_A553_000502DD3AA8__INCLUDED_

#ifdef SWSERVICES_EXPORTS
#define SWSERVICES_API __declspec(dllexport)
#else
#define SWSERVICES_API __declspec(dllimport)
#endif

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Stdafx.h"
#include <string>

class CXMLReg;

/**
 *  Use this class to access the Windows registry in a Server Watch plugin. The class
 *  is designed to help abstract out the complexity of the Windows registry, and its use
 *  allows us to control the data in the registry and insulate your plugin's registry
 *  settings from changes to how the registry is handled in Server Watch.
 *
 *  @par Registry Safety
 *  One common problem with the windows registry is that it is too easy to modify
 *  the wrong area of the registry. This class bypasses that problem by only allowing
 *  the user to operate within a strict subsection of the tree -- the applications
 *  registry space.
 *  
 *  @par Required for use in Plugins
 *  If you do not use this class to store your registry, Server Watch will not be able
 *  to update your registry values during an upgrade of Server Watch and your plugin may
 *  break or become non-backwards compatible. Use of this class is compulsory for all
 *  registry work in Server Watch.
 *  
 *  @par Plugin Examples
 *  Use this class with the strRegistryBase string defined in SWPluginServerStruct
 *  to make sure that you always save plugin data to the right place.
 *  
 *  @code
 *  // Create the registry class
 *  DFRegTool reg;
 *  
 *  // Open registry to the appropriate location in the registry for your plugin.
 *  // The m_pCurrentServer is an instance of SWPluginServerStruct that was passed
 *  // into the plugin.
 *  reg.OpenKey(m_pCurrentServer->strRegistryBase);
 *
 *  // Save a string
 *  reg.AddStr(L"DemoString", L"DemoValue");
 *
 *  // Close the key. This is optional, the instance of DFRegTool will automatically
 *  // close the key when the instance is destroyed.
 *  reg.CloseKey();
 *  @endcode
 */
class SWSERVICES_API DFRegTool  
{
	#pragma warning( push )
	#pragma warning( disable : 4251 )

public:
    
    /** @name Constructor/Destructor */
    //@{
        DFRegTool();
        virtual ~DFRegTool();
    //@}

    /** @name Opening/Closing the key */
    //@{
        LONG OpenKey();
        LONG OpenKey(const CAtlString&);
        LONG OpenKey(const std::wstring&);
        LONG OpenKey(const TCHAR*);
        LONG OpenKey(DWORD&);
        LONG OpenKey(const CAtlString&, DWORD&);
        LONG OpenKey(const std::wstring&, DWORD&);
        LONG OpenKey(const TCHAR*, DWORD&);
        LONG CloseKey();
    //@}

    /** @name Numeric Values 
     *  Stores the numeric value of the key/value pair
     *  as a simple DWORD in the registry. The variable
     *  type of the value can be retrieved as one of a many
     *  types, but the user should take care so as not to 
     *  retrieve a value from the registry and put it into
     *  a variable smaller than required or the value will be
     *  truncated.
     */
    //@{
        LONG AddDword(LPCTSTR key, DWORD value);

        //@{
        /**
         * Retrieves the DWORD value for a specified key in the
         * registry.
         * @param key The key, in relation to the opened location, who's
         *            calue you are attempting to retrieve.
         * @param value The variable to insert the value associated with
         *              the specified key.
         * @return The success value specified by the Windows registry API
         *         for fetching the value.
         */
        LONG GetDword(LPCTSTR key, DWORD& value);
	    LONG GetDword(LPCTSTR key, int& value);
	    LONG GetDword(LPCTSTR key, bool& value);
        //@}
    //@}

    /** @name Basic String Values
     *  Stores the string value of the key/value pair
     *  as a simple human readable unicode string. The
     *  variable type of the value can be one of a many
     *  types.
     */
    //@{
        LONG AddStr(LPCTSTR key, LPCTSTR value);
        LONG AddStr(LPCTSTR key, const CAtlString& value);
        LONG AddStr(LPCTSTR key, const std::wstring& value);
        LONG GetStr(LPCTSTR key, LPTSTR, DWORD value);
        LONG GetStr(LPCTSTR key, CAtlString& value);
        LONG GetStr(LPCTSTR key, std::wstring& value);
    //@}

    /** @name Encoded String 
     *  Stores the string value of the key/value pair
     *  in an encoded format. The encoding is basic and
     *  should not be considered secure. The variable type
     *  of the value can be one of a many types.
     *
     *  @par Code Numbers
     *  The values passed into codeA and codeB when the string
     *  is set must be the same values passed into the call
     *  to GetCodedStr() to fetch the string of the value returned
     *  will not be correct.
     */
    //@{
        LONG AddCodedStr(LPCTSTR key, const CAtlString& value,   int codeA, int codeB);
        LONG AddCodedStr(LPCTSTR key, const std::wstring& value, int codeA, int codeB);
        LONG AddCodedStr(LPCTSTR key, const TCHAR* value,        int codeA, int codeB);
        LONG GetCodedStr(LPCTSTR key, CAtlString& value,         int codeA, int codeB);
        LONG GetCodedStr(LPCTSTR key, std::wstring& value,       int codeA, int codeB);
    //@}

    /** @name Deletion */
    //@{
        LONG DeleteContents();
        LONG DeleteFolder(LPCTSTR);
        LONG DeleteKey(LPCTSTR);
    //@}

    /** @name Replication */
    //@{
        LONG Replicate(const DFRegTool&);
    //@}

    /** @name Tree Iteration */
    //@{
	    LONG NextKey(CAtlString &strFileName, int iSetBack = 0);
	    LONG NextKey(std::wstring &strFileName, int iSetBack = 0);
        LONG NextValue(CAtlString &strValueName, int iSetBack = 0);
        LONG NextValue(std::wstring &strValueName, int iSetBack = 0);
    //@}
    
    /** @name XML Import/Export */
    //@{
        LONG BeginExportToXML();
        LONG ExportKeyToXML(LPCTSTR szKeyName, DFRegTool& regKey);
        LONG EndExportToXML(LPCTSTR szFileName);
        LONG BeginImportFromXML( LPCTSTR szFileName );
        LONG ImportFromXML( LPCTSTR szKeyName, CXMLReg& xmlImport );
        LONG EndImportFromXML();
        CXMLReg* GetXMLImporter();
        CXMLReg* GetXMLExporter();
    //@}

protected:
	enum FetchType
	{
		Readable,
		Encoded,
		Whole
	};

    LONG DeleteHelper(HKEY hParentKey);
    LONG ExportHelper(CXMLReg& xmlFile, HKEY hCurrentKey);
    LONG ImportHelper(CXMLReg& xmlFile, HKEY hCurrentKey);
    LONG ReplicateHelper(HKEY hParentKey, HKEY hCopyFromParentKey);
    LONG GetStrInternal(HKEY hKey, LPCTSTR string, std::wstring &lpszData, FetchType type);
    LONG GetDwordInternal( HKEY hKey, LPCTSTR string, DWORD &dwWord);
    CAtlString BasePath;
    HKEY m_hIp;
    CXMLReg* m_xmlExportFile;
    CXMLReg* m_xmlImportFile;

	#pragma warning( pop )
};

#endif // !defined(AFX_REGTOOL_H__EBCBF331_598F_11D2_A553_000502DD3AA8__INCLUDED_)
