/*
    ISWPluginMenu.h
    Copyright (c) 2000-2004, Deep Fried Software. All rights reserved.
 */

#ifndef ISWPLUGINMENU_H
#define ISWPLUGINMENU_H

#ifdef SWSERVICES_EXPORTS
#define SWSERVICES_API __declspec(dllexport)
#else
#define SWSERVICES_API __declspec(dllimport)
#endif

#include "stdafx.h"
#include "swmacros.h"

struct SWPluginServerStruct;

/**
 * This interface is the plugin interface to Action Menu's in
 * in Server Watch.  An instance of this interface is passed 
 * to the plugin in the CreateMenu function, and the user creates
 * the menu to be displayed using the ISWMenuContext::AddMenuItem
 * function.  The plugin can get context information to aid in the
 * creation of the menu through the ISWMenuContext::GetMenuType and 
 * ISWMenuContext::GetMenuContext functions.
 * When the menu item is triggered, Server Watch calls the 
 * SWMENUACTIONCALLBACK function specified in the fnPtr parameter during
 * the AddMenuItem for that menu item.
 */
class SWSERVICES_API ISWPluginMenu
{
public:
    /** 
     * ISWMenuContext provides context information to aid in the creation of
     * the menu.  An object derived from this interface will be returned from
     * ISWPluginMenu::GetMenuContext() function.
     */
    class ISWMenuContext
    {
    public:
        enum ISWMenuContextType
        {
            MENUROWCOLSTRUCT ///< This ISWMenuContext is an instance of MenuRowColStruct
        };

        /** 
         * GetType returns the type of context object being returned.  This allows a plugin
         * to query the type to know what type of object to dynamic_cast the ISWMenuContextType*
         * to to get context information for the menu.
         */
        virtual ISWMenuContextType GetType() = 0;
    };

    /** Denotes the type of action menu being created or worked with. */
    enum SWMenuType
    {
        // Note: *ALWAYS* add to the end of this enum
        Undefined, 
        AuxStat, ///< The action was triggered from the Auxiliary status field.
        UserList ///< The action was triggered from a cell in the UserList.
    };

    /** 
     * Typedef of the callback function signature used when notifying plugins of a plugin action.
     * The instance of this function type (specificed with the action in AddMenuItem) is called whenever the 
     * user has selected an action 
     * <BR><BR>
     * After the menu is created with AddMenuItem, the menu is displayed to the user.  If the user clicks a menu item
     * then the function associated with the action is executed.
     *
     * @param pThisServer A SWPluginServerStruct that contains the information related
     * to this server.
     * @param menuType The menutype that triggered this action.
     * @param pMenuContext The ISWMenuContext of the location that triggered this action.
     * @retval SW_OK Return this if the processing within this function was successful
     * @retval SW_FAIL Return this if the processing within this function failed
     */
    typedef void ( __cdecl *SWMENUACTIONCALLBACK)(SWPluginServerStruct* pThisServer, ISWPluginMenu::SWMenuType menuType, ISWPluginMenu::ISWMenuContext* pMenuContext);

    /** GetMenuType returns the type of menu being created.  Most likely a different set of menu items should be returned for different contexts */
    virtual SWMenuType GetMenuType() = 0;
    virtual ISWMenuContext* GetMenuContext() = 0; ///< GetMenuContext returns the ISWMenuContext pointer for this menu.

/** 
 * AddMenuItem flags parameter flag.
 * ISWPluginMenu_IsSeparator denotes the menu item is a separator.
 */
#define ISWPluginMenu_IsSeparator  1
/** 
 * AddMenuItem flags parameter flag.
 * ISWPluginMenu_IsDisabled denotes the menu item is disabled.
 */
#define ISWPluginMenu_IsDisabled   2
/** 
 * AddMenuItem flags parameter flag.
 * ISWPluginMenu_IsChecked denotes the menu item should display with a checkbox.
 */
#define ISWPluginMenu_IsChecked    4
    /**
     * AddMenuItem is used to add a menu item to the menu to be displayed.
     * Menu items are displayed in the menu in the order they were added to the menu.
     * @param strItemName The text of the menu item.
     * @param fnPtr Function pointer to the function to call if this menu item is selected.
     * @param flags Bitmask of flags to specify menu properties.
     */
    virtual SWRESULT AddMenuItem( std::wstring strItemName, SWMENUACTIONCALLBACK fnPtr, unsigned int flags ) = 0;

    virtual ~ISWPluginMenu() {}
};

/**
 * @brief Used to give context information of where an action menu is being 
 * created or triggered from when dealing with a ISWPluginMenu::SWMenuType::UserList type of action.
 */
class SWSERVICES_API MenuRowColStruct : public ISWPluginMenu::ISWMenuContext
{
public:

    /** The type of this ISWMenuContext item. */
    virtual ISWPluginMenu::ISWMenuContext::ISWMenuContextType GetType() { return ISWPluginMenu::ISWMenuContext::MENUROWCOLSTRUCT; };

    /**
     * The row of the list control item from which the menu is 
     * being created or the action is being triggered 
     */
    UINT uiRow;

    /**
     * The column of the list control item from which the menu is 
     * being created or the action is being triggered 
     */
    UINT uiCol;
};

#endif //ISWPLUGINMENU_H