/*
    ISWPluginPane.h
    Copyright (c) 2000-2004, Deep Fried Software. All rights reserved.
 */

#ifndef ISWPLUGINPANE_H
#define ISWPLUGINPANE_H

#include <stdafx.h>

/**
 * The ISWPluginPane interface defines a set of functions that, when a class inherits from and
 * properly implements it, will expose the necessary functions functionality to Server Watch so
 * that it can embed your pane into the Server Watch interface. We use the WTL CDialogImpl<>
 * template as a base the windows in our plugins, but you could also use CWindow or CWnd as a
 * base for your windows. This is an example of our CAdminTabDlg class implementing the 
 * ISWPluginPane interface in the header:
 *
 * @code
 * class CAdminTabDlg : 
 *     public CDialogImpl<CAdminTabDlg>,
 *     public ISWPluginPane
 * {
 * public:
 *     CAdminTabDlg();
 *     ~CAdminTabDlg();
 *
 *     enum { IDD = IDD_ADMINTABDLG };
 *
 *     virtual const std::wstring& GetTitle() const;
 *     virtual DisplayMode GetDisplayMode() const;
 *     virtual HWND GetPaneHWND() const;
 *     virtual bool IsResizeable() const;
 *     virtual void PrePaneSelection();
 *
 *     // Other elements of the class go here
 *     ...
 * };
 * @endcode
 *
 * When this interface is properly implemented, your custom pane will be displayable
 * in Server Watch along with the standard server panes like the users list, web
 * output pane, etc.
 * <BR><BR>
 * We suggest that you create your panes in the LoadPlugin() process and destroy them
 * during the UnloadPlugin() process of either the @ref swplugindll.h "standard plugin"
 * interface or the @ref swplugindllex.h "plugin extender" interface. To attach the panes
 * to your server, call ISWServerData::InsertPane() during the InitServer() processing
 * (you will not need to detach them).
 * <BR><BR>
 * You do not need to create an instance of the pane for each server created. The pane
 * will recieve a special windows message (#SWP_SET_DIALOG) each time that the pane is
 * selected for display by the user. This message includes the SWPluginServerStruct
 * associated to the server that you the pane should now handle.
 * <BR><BR>
 * You will need to implement a few Server Watch specific message handlers. The messages are
 * defined in SWMessages.h along with the message values that Server Watch reserves as its own
 * (don't reuse these values or you may have to make changes to your plugin as Server Watch
 * revieves new enhancements in the future.
 *
 * @code
 * BEGIN_MSG_MAP(CAdminTabDlg)
 *     MESSAGE_HANDLER(SWP_SET_DIALOG, OnSetDialog)
 *     MESSAGE_HANDLER(SWP_CHANGES_APPLIED, OnChangesApplied)
 *     MESSAGE_HANDLER(SWP_CHANGES_RESET, OnChangesReset)
 * END_MSG_MAP()
 *
 * // This function is called whenever the pane is displayed to the user or when the
 * // SWPluginServerStruct that is currently associated to the pane changes. The WPARAM
 * // will be a pointer to the SWPluginServerStruct that you should use to display data.
 * // The LPARAM will contain a value that indicates what triggered the call to this
 * // function. See the "SWP_SET_DIALOG Mask" in SWMessages.h for the mask values.
 * LRESULT OnSetDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
 *
 * // The apply button was pressed. You should save the values in the pane.
 * LRESULT OnChangesApplied(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
 *
 * // The reset button was pressed. You should reset the values in the pane to their
 * // previously saved state.
 * LRESULT OnChangesReset(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
 * @endcode
 *
 * You must set the pane to have a style of WS_POPUP. Server Watch will change this style
 * and parent it appropriately. If it does not start as WS_POPUP, Server Watch will fail to
 * attach your pane. You should also turn off the border and title bar entirely. Leaving these
 * on will not stop Server Watch from mounting the pane, but it won't look very good.
 * <BR><BR>
 * If you have a pane with settings, you will need to notify Server Watch when settings on the
 * pane have changed so it can correctly enable and disable the apply and reset buttons. To do
 * this, you send specific messages to the panes parent window. Read the "Message From Plugin
 * Panes" section of SWMessages.h for a complete understanding of these messages. Because the pane
 * will be reparented by Server Watch, it is critical that you call GetParent() for each message
 * you inted to send to the parent window.
 */
class ISWPluginPane
{
public:
    /**
     * The DisplayMode identifies information about the type of pane being handled.
     * This information is used by Server Watch to identify the environment that
     * should be presented when the pane is displayed for the pane to work correctly.
     */
    enum DisplayMode
    {
        DataOnly  = 1,      ///< The pane doesn't need reset or apply buttons
        Settings  = 2,      ///< The pane is a settings dialog (needs reset and apply buttons)
        DataApply = 3       ///< The is a data pane, but it needs reset and apply buttons.
    };

    /** @return The title that should be displayed for the pane */
    virtual const std::wstring& GetTitle() const = 0;

    /** @return The @ref ISWPluginPane::DisplayMode "DisplayMode" that identifies the pane type */
    virtual DisplayMode GetDisplayMode() const = 0;

    /**
     * Returns the Windows GUI handle to the pane that will be hosted in
     * Server Watch. This window should not have a parent (create it with
     * a NULL parent) and it should have a style of WS_POPUP. When Server
     * Watch ataches it to the primary window, the WS_POPUP style will be
     * replaced with WS_CHILD and the pane will be parented to the
     * appropriate window. This is critical to making your pane create and
     * attach correctly.
     * @return The HWND of the Windows pane itself.
     */
    virtual HWND GetPaneHWND() const = 0;

    /** @return Whether your pane was designed to be resized or not. */
    virtual bool IsResizeable() const = 0;

    /**
     * Tells the pane it is about to be selected for display.
     *
     * @deprecated Use the #SWP_SET_DIALOG Windows message for this notification
     * instead as it contains the SWPluginServerStruct that identifies the server
     * that is associated to the new selection.
     */
    virtual void PrePaneSelection() = 0;
};

#endif //ISWPLUGINPANE_H