/*
    ISWServerData.h
    Copyright (c) 2000-2004, Deep Fried Software. All rights reserved.
 */

#if !defined(AFX_RUNTIMEDATAPLUGV1_0_0_H__FDF6EDA2_DE73_11D3_89EF_0040333C2E7E__INCLUDED_)
#define AFX_RUNTIMEDATAPLUGV1_0_0_H__FDF6EDA2_DE73_11D3_89EF_0040333C2E7E__INCLUDED_

#include "ISWPluginPane.h"
#include <string>

#define RSD_OK          0
#define RSD_TIMEOUT     -1
#define RSD_INCOMPLETE  -2

//Icon non-number settings
// -1 is used to say that the app just started
#define ICON_UP         -2
#define ICON_DOWN       -3

enum SWRESULT;

/**
 * @brief Contains the server data that will be outputed to the user and provides an API for
 * retrieving and setting the current server information. The data in this class should be set
 * during the "Ping" process in the plugin. It can be read at any time.
 *
 * Server Watch will generate an instance of this class automatically. You will not need to store
 * instances of this class as the appropriate instance will be passed into the plugin as a member
 * of an SWPluginServerStruct when the plugin is expected to handle a server specific event.
 *
 * @par Thread safety design:
 * This class is used in a highly threaded environment, therefore it is important that
 * users of this class use the concurrency functions of this class correctly. You can read
 * more about the concurrency design in the "Concurrency" section below. If these functions
 * are not used correctly, your plugin will not work as you expect. The concurrency design
 * of this class differs in important ways from ISWServerSettings, so you should understand
 * both.
 *
 * @bug If you pass an invalid table id string into any of the table functions, with the
 * exception of TableExists(), Server Watch will crash.
 */
class ISWServerData
{
    public:

        /** @name Concurrency
        *  @brief Functions that manage the transaction based reader/writer thread safety process
        *  
        *  Unlike the concurency functionality of @ref ISWServerSettings, the concurrency of this
        *  class follows a very modified version of the readers/writers pattern (see @ref ISWServerSettings
        *  for a good description of the classic pattern). It must be possible to read and write
        *  runtime data at the same time.  To do otherwise can cause long delays in the user
        *  interface.  To accomplish this we have created the following design:
        *  <BR><BR>
        *  There is both a read buffer and a write buffer. Standard read functions ("get...", etc) read from the
        *  read buffer.  All write functions ("set...", etc) write into the write buffer. The write buffer is
        *  copied into the read buffer when both of the following conditions are met: 1) EndWrite has been called
        *  and 2) there are no readers.  EndWrite() blocks until there are no readers and the copy of the 
        *  write buffer into the read buffer is complete.
        *  <BR><BR>
        *  What does this mean for every day use?
        *  <BR><BR>
        *  For the most part, everything works as expected. Multiple readers can read concurrently. Only one
        *  writer gains write access at a time, though they don't have to wait for the readers to exit. However,
        *  if a writer sets a value, the following standard fetch on that value will not necessarily retrieve the value
        *  that was set. Only when a call to EndWrite() has been completed will the read buffer be committed
        *  and guaruntee that the value fetched is the same as the value set.
        *  <BR><Br>
        *  If you need to access a value you have set before you can call EndWrite(), it is suggested that you
        *  cash that value locally until you can call EndWrite().  There are some functions designed to fetch data directly
        *  from the write buffer. These functions are desognated by titles in the form of "get...Uncommitted." They 
        *  should be used carfully because missuse of these funtions invalidates the thread safety design of this class.
        *  Specifically, the calling thread of the function must have called @ref ISWServerData::BeginWrite() on this class
        *  and must not have called @ref ISWServerData::EndWrite() yet. In otherwords, <b>the thread must own the write
        *  context.</b> If you do not follow this restriction, you are "off the island" and will likely crash Server Watch
        *  in random ways that are difficult to reproduce and fix.
        *  <BR><BR>
        *  Remember that you only want to call EndWrite() when you have update <b>ALL</b> of the values you intend
        *  to update.  If you do a partial update, another writer could beat you to the next BeginWrite() call and
        *  skew your data, or more likely a reader will come through while you are processing the rest and cause the
        *  display of only partially accurate information.
        *  <BR><BR>
        *  @note Technically, it is possible to fetch data from this class without calling
        *  BeginRead(), but doing so invalidates the data integrity guaranties of the
        *  class and could cause the program to crash. It is not possible to write to
        *  the class without previously calling BeginWrite().
        */
        //@{
            /**
             * @brief Puts the class in read mode.
             *
             * This function blocks if a call to EndWrite() is causing the write buffer to be
             * copied into the read buffer. It will unblock when that copy is complete.
             */
            virtual void BeginRead() =0;

            /** @brief Removes the class from read mode. */
            virtual void EndRead() =0;

            /**
             * @brief Puts the class in write mode.
             *
             * You should update everything you intend to update before calling EndWrite(). Doing
             * partial updates allows other threads to update the class and potential cause it to
             * contain invalid data.
             * <BR><BR>
             * If you have called BeginWrite(), you should refrain from calling read functions.  Writers are allowed
             * to read, but they still read from the read buffer which is unlikely the functionality that you are
             * expecting. This is different than the functionality in @ref ISWServerSettings.
             */
            virtual void BeginWrite() =0;

            /**
             * @brief Removes the class from write mode.
             *
             * This function will block until there are no readers and the write
             * buffer has been completely copied into the read buffer.
             */
            virtual void EndWrite() =0;

            /**
             * @brief Retains the class for use.
             *
             * Call Retain on the class when you spawn a thread. This will allow you to use the class
             * without fear of it being deleted while you are using it.
             * <BR><BR>
             * If you call retain, you must call Release when done with the class. If you do not match
             * each Retain call with a call to release, this class will not be correctly cleaned up
             * when Server Watch is quit.
             */
            virtual void Retain() =0;

            /**
             * @brief Removes your retain rights for the class.
             * 
             * Only call Release if you have called Retain. Calling Release too many times will result in
             * a crash in Server Watch.  So, make sure your Release and Retain match up one-for-one.
             */
            virtual void Release() =0;
        //@}

        /** @name Basic Runtime Server Values */
        //@{
            /**
             * @brief Sets the name of the server as reported by the server.
             * 
             * This is not the same as the UserSetServerName() defined in ISWServerSettings. That
             * name is chosen by the user of Server Watch.
             * 
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetServerName( std::wstring name ) =0;

            /**
             * @brief The name of the server as reported by the server.
             *
             * This is not the same as the UserSetServerName() defined in ISWServerSettings. That
             * name is chosen by the user of Server Watch.
             *
             * @return The retrieved server name.
             */
            virtual std::wstring GetServerName() const =0;

            /**
             * @brief Sets the reported maximum number of users that can be on the server.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetMaxUsers( int users ) =0;

            /**
             * @brief Retrieves the maximum number of user that can be on the server
             *
             * @return The maximum number of users.
             */
            virtual int GetMaxUsers() const =0;

            /**
             * @brief Sets the reported current number of users that are on the server.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetTotalUsers( int users ) =0;

            /**
             * @brief Retrieves the current number of users on the server.
             *
             * @return The number of users on the server.
             */
            virtual	int GetTotalUsers() const =0;

            /**
             * @brief Sets the text to be diaplyed in the AuxStat box.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetAuxStat( std::wstring theStat ) =0;

            /**
             * @brief The text to be displayed in the AuxStat box.
             * 
             * @return The AuxStat text.
             */
            virtual std::wstring GetAuxStat() const =0;

            /**
             * @brief Sets the text to be displayed as the "status" display in Server Watch.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetStatusText( std::wstring theStatus ) =0;

            /**
             *  @brief Returns the text that will be displayed in the "status" display
             *  box.
             *
             *  @return The "status" text.
             */
            virtual std::wstring GetStatusText() const =0;
        //@}

        /**
         * @name Tray Icon
         * The plugin manages the values that will be displayed in the tray icon for monitored
         * servers. The icon itself is set by calling SetIconNumber(), SetIconUp(), or SetIconDown().
         * A call to any one of these functions will override the previous setting.
         * @code
         * // pData is a pointer to an instance of ISWServerData
         * 
         * pData->BeginWrite();
         * pData->SetIconNumber(10);
         * pData->EndWrite();
         * // Icon now displays 10
         *
         * pData->BeginWrite();
         * pData->SetIconUp();
         * pData->SetIconNumber(20);
         * pData->EndWrite();
         * // Icon now displays 20
         *
         * pData->BeginWrite();
         * pData->SetIconDown();
         * pData->EndWrite();
         * // Icon now displays down symbol
         * @endcode
         */
        //@{
            /**
             * @brief Sets the icon number to be desplayed by Server Watch.
             *
             * Only the values 0 - 9,999 are supported.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetIconNumber( int num ) =0;

            /**
             * @brief The number to be displayed in the tray icon.
             *
             * @return The icon number to be displayed in the tray.
             */
            virtual int GetIconNumber() const =0;

            /**
             * @brief Sets the icon to display the "up" icon.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetIconUp() =0;

            /**
             * @brief Sets the icon to display the "down" icon.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetIconDown() =0;

            /**
             * @brief Sets the text to be displayed as the "tip" for the tray icon.
             *
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetIconTip( std::wstring theTip ) =0;

            /**
             * @brief Returns the text to be displayed as the "tip" for the tray icon.
             *
             * @return The tray icon "tip" text.
             */
            virtual std::wstring GetIconTip() const =0;
        //@}

        /**
         * @name Data Tables
         * Each data table is displayed in Server Watch as an individual server pane. You can specify one of these
         * tables to be the User Table.  This table has special significance to the system.  For example, its first
         * column will be searched for user set "buddies" that may be on the server. If the buddy is found, a special
         * column in the display of the Users Table will display that that individual is a buddy and alerts will be
         * triggered.
         */
        //@{
            /**
             * @brief Creates a new table identified by the string strTableID.
             *
             * The tables will be displayed in the order created.
             *
             * @param strTableID The name of the table to be created
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             * @retval SW_INVALIDARG Requested table id is already in use
             */
            virtual SWRESULT CreateTable( const std::wstring & strTableID ) =0;

            /**
             * @brief Tests whether or not a specified table exists.
             *
             * @param strTableID The name of the table
             * @return Whether or not the requested table exists
             */
            virtual const bool TableExists( const std::wstring & strTableID ) const =0;

            /**
             * @brief Retrieves the name of a specified table.
             *
             * The name of the table differes from the identifying string.
             * @param strTableID The name of the table
             * @return The name of the table
             */
            virtual const std::wstring & GetTableName( const std::wstring & strTableID ) const =0;

            /**
             * @brief Sets the number of a specified table
             *
             * The name of the table differes from the identifying string.
             * @param strTableID The name of the table
             * @param strName The new name for the table
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetTableName( const std::wstring & strTableID, const std::wstring & strName ) =0;

            /**
             * @brief Sets ID that will be used as the users table
             *
             * The primary difference between the users table and a regular table, is that the first column of the
             * users table will be used for name searching during buddy identification. When displayed, the GUI will
             * place an extra column indicator to display whether of not each row was identified as a buddy.
             * @param strName The ID that identifies teh users table
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetUsersTableID( const std::wstring & strName ) =0;

            /**
             * @brief Retrieves the ID that will be used as the users table
             *
             * The primary difference between the users table and a regular table, is that the first column of the
             * users table will be used for name searching during buddy identification. When displayed, the GUI will
             * place an extra column indicator to display whether of not each row was identified as a buddy.
             * @return The ID that will be used as the users table.
             */
            virtual const std::wstring & GetUsersTableID() const =0;

            /**
             * @brief Sets the number of columns on a specified table
             *
             * @param strTableID The name of the table
             * @param NumCols The number of columns the table will have
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetNumCols( const std::wstring & strTableID, int NumCols ) =0;

            /**
             * @brief Retrieves the number of columns on a specified table
             *
             * @param strTableID The name of the table
             * @return The number of columns the table has
             */
            virtual int GetNumCols( const std::wstring & strTableID ) const =0;

            /**
             * @brief Sets the name to be displayed for a table on a specified table
             *
             * @param strTableID The name of the table
             * @param index The numeric id of the column (0 starts as the first column)
             * @param title The new title
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             * @retval SW_OUTOFBOUNDS The index was invalid
             */
            virtual SWRESULT SetColHeaderTitle( const std::wstring & strTableID, int index, const std::wstring &title ) =0;

            /**
             * @brief Retrieves the name of a column on a specified table
             *
             * @param strTableID The name of the table
             * @param index The numeric id of the column (0 starts as the first column)
             * @param title The string that will be set to the title if the function is successful
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_OUTOFBOUNDS The index was invalid. The variable passed into parameter "title" was not changed
             */
            virtual SWRESULT GetColHeaderTitle( const std::wstring & strTableID, int index, std::wstring &title ) const =0;

            /**
             * @brief Retrieves the default width of a column on a specified table
             *
             * The default width is the width that the column starts with before the user
             * modifies it.
             * @param strTableID The name of the table
             * @param index The numeric id of the column (0 starts as the first column)
             * @param width The int that will be set to the width if the function is successful
             * @return SWResult defining the success of the function<BR>
             * @retval SW_OK The function completed successfully
             * @retval SW_OUTOFBOUNDS The index was invalid. The variable passed into parameter "width" was not changed
             */
            virtual SWRESULT GetColDefaultWidth( const std::wstring & strTableID, int index, int &width ) const =0;

            /**
             * @brief Sets the number of columns on a specified table
             *
             * The default width is the width that the column starts with before the user
             * modifies it.
             * @param strTableID The name of the table
             * @param index The numeric id of the column (0 starts as the first column)
             * @param width The width to set the column to
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             * @retval SW_OUTOFBOUNDSThe index was invalid
             */
            virtual SWRESULT SetColDefaultWidth( const std::wstring & strTableID, int index, int width ) =0;

            /**
             * @brief Forces all columns on a table to have a current width that is the same as their default width.
             *
             * It is not common for a plugin to call this. The historical reason for its inclusion in the plugin
             * API is for the extremely rare occasion when all the columns of a table are removed because some
             * value has changed on the server (version number, for instance), and all new columns were added.
             * At that time the old columns' widths would still be in place which would usually look bad to the
             * user.
             *
             * @param strTableID The name of the table
             */
            virtual SWRESULT SetColWidthToDefault( const std::wstring & strTableID ) =0;

            /**
             * @brief Retrieves the number of rows on a specified table.
             *
             * This function returns the commited value. If you need the value for the uncommited
             * state, use the Uncommited version of this function.
             * @param strTableID The name of the table
             * @return The number of rows the table has
             * @sa GetNumRowsUncommitted()
             */
            virtual int GetNumRows( const std::wstring & strTableID ) const =0;

            /**
             * @brief Retrieves the uncommited number of rows on a specified table.
             * 
             * This function returns the uncommited value. If you need the value for the commited
             * state, use the standard version of this function.
             * @param strTableID The name of the table
             * @return The number of rows the table has
             * @ref GetNumRowsUncommitted()
             */
            virtual int GetNumRowsUncommitted( const std::wstring & strTableID ) const =0;

            /**
             * @brief Removes all the rows from a specified table.
             * 
             * All row data will be reset, however, the columns will remain intact. 
             * @param strTableID The name of the table
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT ResetData( const std::wstring & strTableID ) =0;

            /**
             * @brief Adds a new row to a specified table.
             * 
             * The row is added to the end of the table.
             * @param strTableID The name of the table
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT AddRow( const std::wstring & strTableID ) =0;

            /**
             * @brief Sets the value of a specified table cell.
             * 
             * A table cell is the intersection of a row and column in a table. Any string value is considered legal.
             * @param strTableID The name of the table
             * @param col The column that, when combined with the row, defines the table cell
             * @param row The row that, when combined with the column, defines the table cell
             * @param data The string value for the cell
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT SetData( const std::wstring & strTableID, int col, int row, const std::wstring &data ) =0;

            /**
             * @brief Retrieves the value of a specified table cell.
             * 
             * A table cell is the intersection of a row and column in a table.
             * <BR><BR>
             * This function returns the commited value. If you need the value for the uncommited
             * state, use the Uncommited version of this function.
             * @param strTableID The name of the table
             * @param col The column that, when combined with the row, defines the table cell
             * @param row The row that, when combined with the column, defines the table cell
             * @param data A string that will contain the value for the cell
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             */
            virtual SWRESULT GetData( const std::wstring & strTableID, int col, int row, std::wstring &data ) const =0;


            /**
             * @brief Retrieves the uncommited value of a specified table cell.
             * 
             * A table cell is the intersection of a row and column in a table.
             * <BR><BR>
             * This function returns the uncommited value. If you need the value for the commited
             * state, use the standard version of this function.
             * @param strTableID The name of the table
             * @param col The column that, when combined with the row, defines the table cell
             * @param row The row that, when combined with the column, defines the table cell
             * @param data A string that will contain the value for the cell
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT GetDataUncommitted( const std::wstring & strTableID, int col, int row, std::wstring &data ) const =0;
        //@}

        /**
         * @name Custom Panes
         * Server Watch allows plugins to define custom graphical panes.
         * @sa ISWPluginPane
         */
        //@{
            /**
             * Inserts a class that implements ISWPluginPane into the list of panes
             * related to the server.
             * @return SWResult defining the success of the function
             * @retval SW_OK The function completed successfully
             * @retval SW_INCORRECTSTATE The class is not set to write mode with a call to BeginWrite()
             */
            virtual SWRESULT InsertPane( ISWPluginPane* pane ) =0;
        //@}
};
#endif // !defined(AFX_RUNTIMEDATAPLUGV1_0_0_H__FDF6EDA2_DE73_11D3_89EF_0040333C2E7E__INCLUDED_)
