/*
    SWError.h
    Copyright (c) 2000-2004, Deep Fried Software. All rights reserved.
 */

#if !defined(AFX_SWERROR_H__4812BCB5_15D1_41BA_B301_875749CA0448__INCLUDED_)
#define AFX_SWERROR_H__4812BCB5_15D1_41BA_B301_875749CA0448__INCLUDED_

#ifdef SWSERVICES_EXPORTS
#define SWSERVICES_API __declspec(dllexport)
#else
#define SWSERVICES_API __declspec(dllimport)
#endif

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <memory>

/**
 * Posts a new error message to the Server Watch error console.
 * @param strCaption The error heading text. This usually defines the
 * topic the error fits under.
 * @param strMessage The text of the actual error.
 */
#define SWERROR_MESSAGE(strCaption, strMessage) \
    do{ SWError::GetInstance()->SetError(strCaption, strMessage ); \
    } while (0)

/**
 * Posts a new error message to the Server Watch error console if the
 * plugin was compiled in debug mode.
 * @param strCaption The error heading text. This usually defines the
 * topic the error fits under.
 * @param strMessage The text of the actual error.
 */
#define SWDEBUG_ERROR(strCaption, strMessage) \
			SWDEBUG_ERROR_IMPL(strCaption, strMessage)

#ifndef _DEBUG
    #define SWDEBUG_ERROR_IMPL(strCaption, strMessage) ((void)0)
#else
    #define SWDEBUG_ERROR_IMPL(strCaption, strMessage) \
        do{ SWError::GetInstance()->SetError(strCaption, strMessage ); \
        } while (0)
#endif

class CErrorDlg;
class SWSERVICES_API CErrorDestroyer;

/**
 * @brief The SWError class allows you to post error messages to the Server Watch error console.
 
 * The Server Watch error console displays all the errors it recieves in a listed format. The
 * error console should be used instead of calls to MessageBox in most cases for the following reason:
 * <BR><BR>
 * 1) Message boxes can stop Server Watch from continuing standard processing. Since Server Watch is
 * often set to run regularly in the background, this will cease the tool from working as the user
 * expects.<BR>
 * 2) Errors will often occur during the ping process which can be run automatically at intervals.
 * Traditional error dialogs will often be be created in mass, causing the user to close large
 * numbers of error dialogs when they return to their computer.<BR>
 * 3) Most traditional error message dialogs (like MessageBox) do not have an easy way to cut
 * and paste the error message. This can be a very useful capability in debugging your user's
 * problem.
 * 
 * @par Use the Pre-defined Macros
 * Rarely should you use this class directly. Instead, us the following macros:
 * <BR><BR>
 * #SWERROR_MESSAGE <BR>
 * #SWDEBUG_ERROR
 *
 * @par Singleton
 * SWError is a singleton class and cannot be directly implemented. Instead, to get
 * access to the class, furst call the SWError::GetInstance() static method. This will
 * retrieve the singleton instance.
 *
 * @par Thread Safety
 * SWError is a fully thread safe class. You can use it during Ping processing or in
 * other cases where threading is involved.
 */
class SWSERVICES_API SWError  
{
public:
    /**
     * Retrieves the singleton instance of the SWError class.
     * <BR><BR>
     * You should not use this directly. Instead used on of the many macros
     * that have been defined to handle error handling.
     * @return The singleton instance of the SWError class.
     */
	static SWError* GetInstance();

    /**
     * Posts a new error message to the Server Watch error console.
     * <BR><BR>
     * You should not use this directly. Instead used on of the many macros
     * that have been defined to handle error handling.
     * @param szCaption The error heading text. This usually defines the topic
     * the error fits under.
     * @param szErrorText The text of the actual error.
     */
	void SetError(LPCTSTR szCaption, LPCTSTR szErrorText);

    /**
     * Sets the parent window of the Server Watch error console.
     * @note This function should not be called by plugins.
     * @param hwndNewParent The new parent window of the error console.
     */
    void SetParent(HWND hwndNewParent );

protected:
	SWError();
	SWError(const SWError&);
	virtual ~SWError();

private:
	HWND m_hwndParent;
	CErrorDlg* m_Dlg;
    bool m_bNonVisibleErrorsPosted;

    typedef class SWSERVICES_API std::auto_ptr<SWError> CErrorCleanupClass;
	friend CErrorCleanupClass;
	static CErrorCleanupClass m_instance;
};

#endif // !defined(AFX_SWERROR_H__4812BCB5_15D1_41BA_B301_875749CA0448__INCLUDED_)
