/*
    SWHandle.h
    Copyright 1997-2000, Deep Fried Software. All rights reserved
*/

#if !defined(AFX_SWHANDLE_H__BD650303_DA57_11D3_89EF_0040333C2E7E__INCLUDED_)
#define AFX_SWHANDLE_H__BD650303_DA57_11D3_89EF_0040333C2E7E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef SWSERVICES_EXPORTS
#define SWSERVICES_API __declspec(dllexport)
#else
#define SWSERVICES_API __declspec(dllimport)
#endif

#define SWHANDLE_LENGTH 40

#include "stdafx.h"
#include <string>

/**
 *  @brief This class represents a globally unique identifier.
 *
 *  Instances of this class are used to uniquely identify objects from
 *  one another. A plugin user is unlikely to need to create or use 
 *  instances of SWHandle, but it will be seen heavily throughout the
 *  framework and may appear in demo plugins.
 *  <BR><BR>
 *  @par GUID String Format:
 *  SWHandle uses a specific GUID format string similar to that used in
 *  Windows. All character strings that are used with SWHandles should
 *  Conform to this format:
 *  <BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX<BR>
 *  Where 'X' can be either a letter or number and '-' must be a hyphen.
 */

class SWSERVICES_API SWHandle
{
    #pragma warning( push )
    #pragma warning( disable : 4251 )

public:
    /** @name Constructors/Destructors
     *  SWHandle can be created from scratch or by passing another SWHandle into
     *  the constructor or passing a buffer of wchar_t or char that is compatible
     *  with the GUID format used by SWHandle into the constructor.
     */
    //@{
        /** @brief Standard Constructor
        *  Internal GUID is generated automatically
        */
        SWHandle();
        /** @brief Constructor
        *  Creates internal GUID from the string that was passed in.
        */
        SWHandle( const wchar_t *foo );
        /** @brief Constructor
        *  Creates internal GUID from the string that was passed in.
        */
        SWHandle( const char *foo );
        /** @brief Copy Constructor
        *  Created as a copy of the passed in SWHandle.
        */
        SWHandle( const SWHandle &toCopy);
        /** Destructor */
        virtual ~SWHandle();
    //@}

    /** @name Equal Operators
     *  SWHandle can be set equal to another SWHandle or to number of different
     *  types. The strings must comform to the GUID format used by SWHandle
     */
    //@{
        /** Sets the internal GUID based on the string that was passed in. */
        SWHandle operator=(const CAtlString &foo);
        
        /** Sets the internal GUID based on the string that was passed in. */
        SWHandle operator=(const std::wstring &foo);
        
        /** Sets the internal GUID based on the string that was passed in. */
        SWHandle operator=(const wchar_t *foo);
        
        /** Sets the internal GUID based on the string that was passed in. */
        SWHandle operator=(const char *foo);
        
        /** Sets the internal GUID based on the SWHandle that was passed in. */
        SWHandle operator=(const SWHandle &foo);
    //@}

    /** @name Conversion Operations
     *  The internal string representation of the SWHandle GUID can be retrieved
     *  as a character string.
     */
    //@{
        /** Returns a pointer to the internal GUID */
        operator const char*() const;
        /** Returns a pointer to the internal GUID */
        const char* toCharString() const;
    //@}

    friend SWSERVICES_API bool operator==(const SWHandle &s1, const SWHandle &toCompare);
    friend SWSERVICES_API bool operator==(const SWHandle &s1, const CAtlString &toCompare);
    friend SWSERVICES_API bool operator==(const SWHandle &s1, const std::wstring &toCompare);
    friend SWSERVICES_API bool operator==(const SWHandle &s1, const wchar_t *foo);
    friend SWSERVICES_API bool operator==(const SWHandle &s1, const char *foo);
    friend SWSERVICES_API bool operator!=(const SWHandle &s1, const SWHandle &toCompare);
    friend SWSERVICES_API bool operator!=(const SWHandle &s1, const CAtlString &toCompare);
    friend SWSERVICES_API bool operator!=(const SWHandle &s1, const std::wstring &toCompare);
    friend SWSERVICES_API bool operator!=(const SWHandle &s1, const wchar_t *foo);
    friend SWSERVICES_API bool operator!=(const SWHandle &s1, const char *foo);
    friend SWSERVICES_API bool operator<(const SWHandle &s1, const SWHandle &toCompare);
    friend SWSERVICES_API bool operator<(const SWHandle &s1, const CAtlString &toCompare);
    friend SWSERVICES_API bool operator<(const SWHandle &s1, const std::wstring &toCompare);
    friend SWSERVICES_API bool operator<(const SWHandle &s1, const wchar_t *foo);
    friend SWSERVICES_API bool operator<(const SWHandle &s1, const char *foo);
    friend SWSERVICES_API bool operator<=(const SWHandle &s1, const SWHandle &toCompare);
    friend SWSERVICES_API bool operator<=(const SWHandle &s1, const CAtlString &toCompare);
    friend SWSERVICES_API bool operator<=(const SWHandle &s1, const std::wstring &toCompare);
    friend SWSERVICES_API bool operator<=(const SWHandle &s1, const wchar_t *foo);
    friend SWSERVICES_API bool operator<=(const SWHandle &s1, const char *foo);
    friend SWSERVICES_API bool operator>(const SWHandle &s1, const SWHandle &toCompare);
    friend SWSERVICES_API bool operator>(const SWHandle &s1, const CAtlString &toCompare);
    friend SWSERVICES_API bool operator>(const SWHandle &s1, const std::wstring &toCompare);
    friend SWSERVICES_API bool operator>(const SWHandle &s1, const wchar_t *foo);
    friend SWSERVICES_API bool operator>(const SWHandle &s1, const char *foo);
    friend SWSERVICES_API bool operator>=(const SWHandle &s1, const SWHandle &toCompare);
    friend SWSERVICES_API bool operator>=(const SWHandle &s1, const CAtlString &toCompare);
    friend SWSERVICES_API bool operator>=(const SWHandle &s1, const std::wstring &toCompare);
    friend SWSERVICES_API bool operator>=(const SWHandle &s1, const wchar_t *foo);
    friend SWSERVICES_API bool operator>=(const SWHandle &s1, const char *foo);

    friend SWSERVICES_API std::ostream& operator<< ( std::ostream& os, SWHandle& handle );
    friend SWSERVICES_API std::stringstream& operator<< ( std::stringstream& os, SWHandle& handle );
    friend SWSERVICES_API std::wstringstream& operator<< ( std::wstringstream& os, SWHandle& handle );

private:
    char m_szData[SWHANDLE_LENGTH];
    void validate( char* toValidate );
    bool validateChar( char toValidate );

    #pragma warning( pop )
};

/** @name SWHandle Comparison Operators
 *  A set of comparison functions to allow for easy comparison between
 *  multiple SWHandles or an SWHandle and a compatible type
 */
//@{
    /** Compares two SWHandles for equality */
    SWSERVICES_API bool operator==(const SWHandle &s1, const SWHandle &toCompare);
    /** Compares an SWHandle and a string for equality */
    SWSERVICES_API bool operator==(const SWHandle &s1, const CAtlString &toCompare);
    /** Compares an SWHandle and a string for equality */
    SWSERVICES_API bool operator==(const SWHandle &s1, const std::wstring &toCompare);
    /** Compares an SWHandle and a string for equality */
    SWSERVICES_API bool operator==(const SWHandle &s1, const wchar_t *toCompare);
    /** Compares an SWHandle and a string for equality */
    SWSERVICES_API bool operator==(const SWHandle &s1, const char *toCompare);
    /** Compares two SWHandles for inequality */
    SWSERVICES_API bool operator!=(const SWHandle &s1, const SWHandle &toCompare);
    /** Compares an SWHandle and a string for inequality */
    SWSERVICES_API bool operator!=(const SWHandle &s1, const CAtlString &toCompare);
    /** Compares an SWHandle and a string for inequality */
    SWSERVICES_API bool operator!=(const SWHandle &s1, const std::wstring &toCompare);
    /** Compares an SWHandle and a string for inequality */
    SWSERVICES_API bool operator!=(const SWHandle &s1, const wchar_t *toCompare);
    /** Compares an SWHandle and a string for inequality */
    SWSERVICES_API bool operator!=(const SWHandle &s1, const char *toCompare);
    /** Compares two SWHandles to see if the first is less than the second */
    SWSERVICES_API bool operator<(const SWHandle &s1, const SWHandle &toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second */
    SWSERVICES_API bool operator<(const SWHandle &s1, const CAtlString &toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second */
    SWSERVICES_API bool operator<(const SWHandle &s1, const std::wstring &toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second */
    SWSERVICES_API bool operator<(const SWHandle &s1, const wchar_t *toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second */
    SWSERVICES_API bool operator<(const SWHandle &s1, const char *toCompare);
    /** Compares two SWHandles to see if the first is less than the second or if they are equal */
    SWSERVICES_API bool operator<=(const SWHandle &s1, const SWHandle &toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second or if they are equal */
    SWSERVICES_API bool operator<=(const SWHandle &s1, const CAtlString &toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second or if they are equal */
    SWSERVICES_API bool operator<=(const SWHandle &s1, const std::wstring &toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second or if they are equal */
    SWSERVICES_API bool operator<=(const SWHandle &s1, const wchar_t *toCompare);
    /** Compares an SWHandle and a string to see if the first is less than the second or if they are equal */
    SWSERVICES_API bool operator<=(const SWHandle &s1, const char *toCompare);
    /** Compares two SWHandles to see if the first is greater than the second */
    SWSERVICES_API bool operator>(const SWHandle &s1, const SWHandle &toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second */
    SWSERVICES_API bool operator>(const SWHandle &s1, const CAtlString &toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second */
    SWSERVICES_API bool operator>(const SWHandle &s1, const std::wstring &toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second */
    SWSERVICES_API bool operator>(const SWHandle &s1, const wchar_t *toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second */
    SWSERVICES_API bool operator>(const SWHandle &s1, const char *toCompare);
    /** Compares two SWHandles to see if the first is greater than the second or if they are equal */
    SWSERVICES_API bool operator>=(const SWHandle &s1, const SWHandle &toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second or if they are equal */
    SWSERVICES_API bool operator>=(const SWHandle &s1, const CAtlString &toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second or if they are equal */
    SWSERVICES_API bool operator>=(const SWHandle &s1, const std::wstring &toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second or if they are equal */
    SWSERVICES_API bool operator>=(const SWHandle &s1, const wchar_t *toCompare);
    /** Compares an SWHandle and a string to see if the first is greater than the second or if they are equal */
    SWSERVICES_API bool operator>=(const SWHandle &s1, const char *toCompare);
//@}

/** @name SWHandle stream handlers
 *  A set of stream operator overloads to improve the streaming process
 */
//@{
    /** Streams an SWHandle into an ostream */
    SWSERVICES_API std::ostream& operator<< ( std::ostream& os, SWHandle& handle );
    /** Streams an SWHandle into a stringstream */
    SWSERVICES_API std::stringstream& operator<< ( std::stringstream& os, SWHandle& handle );
    /** Streams an SWHandle into a wstringstream */
    SWSERVICES_API std::wstringstream& operator<< ( std::wstringstream& os, SWHandle& handle );
//@}

#endif // !defined(AFX_SWHANDLE_H__BD650303_DA57_11D3_89EF_0040333C2E7E__INCLUDED_)
