/*
    SWMessages.h
    Copyright 1997-2004, Deep Fried Software. All rights reserved
*/
 
#ifndef _MESSAGES_H
#define _MESSAGES_H

#include "swhandle.h"
/**
    @file SWMessages.h
    @brief The file defines all the windows messages and resource string table IDs that
    will be used by Server Watch.

    @par Plugin Message Definition Restrictions
    Some of these messages are used to interact with the plugin while others are used
    to notify a plugin defined pane of events. Server Watch reserves the right to use
    (WM_USER+1) through (WM_USER+300). If you create a conflicting message that is sent
    to your pane, you may have problems in the future when Server Watch is enhanced.

    @par Plugin Resource ID Restrictions
    Some of these messages are used to interact with the plugin while others are used
    to notify a plugin defined pane of events. Server Watch reserves the right to use
    (WM_USER+1) through (WM_USER+250). If you create a conflicting message that is sent
    to your pane, you may have problems in the future when Server Watch is enhanced.

    @see SWPluginServerStruct
*/

/**
 * @name Plugin Messages
 * Messages used to communicate with the plugins directly. These should not be triggered
 * by the plugin directly or you may recieve unexpected results.
 */
//@{
#define SWP_LOAD_PLUGIN       WM_USER + 10 ///< Internal Plugin Use Only
#define SWP_UNLOAD_PLUGIN     WM_USER + 11 ///< Internal Plugin Use Only
#define SWP_INITSERVER        WM_USER + 12 ///< Internal Plugin Use Only
#define SWP_CLEANUPSERVER     WM_USER + 13 ///< Internal Plugin Use Only
#define SWP_JOINSERVER        WM_USER + 14 ///< Internal Plugin Use Only
#define SWP_MENU_CREATE       WM_USER + 15 ///< Internal Plugin Use Only
// history (pre 3.5): SWP_MENU_ACTION == WM_USER + 16
#define SWP_DO_YOU_SUPPORT    WM_USER + 17 ///< Internal Plugin Use Only
#define SWP_DO_YOU_EXTEND     WM_USER + 18 ///< Internal Plugin Use Only
//@}

/**
 * @name Message to the Plugin Panes
 * Messages used to communicate with the plugins directly. These should not be triggered
 * by the plugin directly or you may recieve unexpected results.
 */
//@{
#define SWP_SET_DIALOG        WM_USER + 100 ///< Notifies a pane that it has just been selected. See SWP_SET_DIALOG Mask
#define SWP_CHANGES_APPLIED   WM_USER + 101 ///< Notifies a pane that the apply button has been pressed so that the pane can act accordingly.
#define SWP_CHANGES_RESET     WM_USER + 102 ///< Notifies a pane that the reset button has been pressed so that the pane can act accordingly.
//@}

/**
 * @name Message From Plugin Panes
 * These messages are sent by the plugin pane to its parent window to notify Server Watch of
 * specific events in the window.
 *
 * @code
 * class MyPane : public ISWPluginPane
 * {
 *   // Implementation of the rest of the class would occur here
 *   ...
 *
 *   void MyDataHasChanged()
 *   {
 *      // Notify Server Watch that data is ready
 *      ::SendMessage( GetParent(), SWPN_DATAREADY, NULL, NULL );
 *
 *      // Notify Server Watch to enable the apply and rest buttons
 *      ::SendMessage( GetParent(), SWPN_ENABLE_APPLY, NULL, NULL );
 *      ::SendMessage( GetParent(), SWPN_ENABLE_RESET, NULL, NULL );
 *   }
 * };
 * @endcode
 */
//@{
#define SWPN_DATAREADY      WM_USER + 200 ///< When called on a panes parent, notifies server watch
                                          ///< that data has changed on the pane.
#define SWPN_ENABLE_APPLY   WM_USER + 201 ///< When called on a panes parent, notifies server watch
                                          ///< that the apply button should be enabled.
#define SWPN_ENABLE_RESET   WM_USER + 202 ///< When called on a panes parent, notifies server watch
                                          ///< that the reset button should be enabled.
//@}


/**
 * @name SWP_SET_DIALOG Mask
 * These are defines that can be used as a bitmask to determine what portions
 * of dialogs are to be updated. These are used in SWP_SET_DIALOG and in the program
 * are also used for IDM_UPDATE_PANE.
 * Data is runtime data (stuff that changes when you ping a server)
 * Settings, are changes that are made to settings.
 * This is so that if the server pings while some data has been changed, those
 * changes aren't lost when the pane updates the new data from the ping
 */
//@{
#define SW_UPDATE_DATA      1 ///< The currently associated ISWServerData has changed or values in it have changed.
#define SW_UPDATE_SETTINGS  2 ///< The currently associated ISWServerSettings has changed or values in it have changed.
#define SW_UPDATE_PARTIAL   4 ///< The values in the currently associated ISWServerData have partially updated. This is not a completely support feature and third-party plugins will not see it.
//@}

/**
 * @name Resource IDs
 *
 * Windows resource id's defined by Server Watch. Define a resource element for each ID
 * defined here. Each one will report important information to Server Watch about the plugin.
 * All resource elements should be strings in the plugin's string table with the single
 * exception of IDI_PLUGIN_ICON which should be an icon resource with at least one 16x16 icon
 * defined for it (32x32 is optional).
 * <BR><BR>
 * String table IDs 1-250 have been reserved for Server Watch. If you use an ID within this range
 * you have have problems with future version of Server Watch.
 */
//@{
#define IDS_PLUGIN_NAME          1   ///< The name of the plugins to be display in teh plugin list.
#define IDS_PLUGIN_VERSION       2   ///< The version of the plugin.
#define IDS_PLUGIN_GUID          3   ///< The GUID that defines the plugin.
#define IDS_PLUGIN_DEFAULT_PORT  4   ///< The default port for this server type.
#define IDS_PROTOCOL_VERSION     6   ///< Always use "3.0" (without the quotes)
#define IDS_CURRENT_VERSION_URL  7   ///< Create, but Leave blank
#define IDS_UPDATE_URL           8   ///< Create, but Leave blank
#define IDS_COPYRIGHT_DATE       9   ///< Copyright for the plugin.
#define IDS_AUX_STAT_TITLE       20  ///< The text to use for the auxiliary stat on the main server pane.
#define IDS_AUTHOR_URL           40  ///< The url for more information about the plugin.
#define IDS_AUTHOR_COMPANY       41  ///< The organization who owns/develops the plugin.
#define IDS_AUTHOR_EMAIL         42  ///< Email contact information for the plugin.
#define IDS_AUTHOR_NAME          43  ///< The name of the author or authors of the plugin.
#define IDI_PLUGIN_ICON          120 ///< The plugin
//@}

#endif // _MESSAGES_H